/* Fesslix - Stochastic Analysis
 * Copyright (C) 2010-2016 Wolfgang Betz
 *
 * Fesslix is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Fesslix is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Fesslix.  If not, see <http://www.gnu.org/licenses/>. 
 */

#define fesslix_flxfem_GegenbauerPolyH_hard_CPP

#include "flxfem_GegenbauerPolyH_hard.h"
#include "flxVec.h"

void hGBP_check_order(const tuint req_order)
{
  if (req_order>MAXHGEGENBAUER) {
    std::ostringstream ssV;
    ssV << "Hierarchic Gegenbauer polynomials are only avaliable for an order up to " << MAXHGEGENBAUER << " and not up to the requested order" << req_order << ".";
    throw FlxException("hGBP_check_order", "The required polynomial order is too large.", ssV.str() );
  }
}


inline const tdouble hGBP0(const tdouble& x)
{
  return (1.0-x)/2.0;
}

inline const tdouble hGBP1(const tdouble& x)
{
  return (1.0+x)/2.0;
}

// -----------------------------------------------------------------------
// the following section has been automatically created using Fesslix
// -----------------------------------------------------------------------
// default flxoutput::float::prec = 20;
// print_hgegenbauerpoly(45);
// -----------------------------------------------------------------------

inline const tdouble hGBP2(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*0.96824583655185425535;
}

inline const tdouble hGBP3(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*2.5617376914898999374*x;
}

inline const tdouble hGBP4(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*(5.8696784409369477942*x2-0.83852549156242117689);
}

inline const tdouble hGBP5(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*(12.744484100974821317*x2-4.2481613669916074016)*x;
}

inline const tdouble hGBP6(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*((26.941097860077082515*x2-14.695144287314771958)*x2+0.81639690485082072602);
}

inline const tdouble hGBP7(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*((56.082367403612245482*x2-43.140282618163269035)*x2+5.8827658115677179396)*x;
}

inline const tdouble hGBP8(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*(((115.61676226989507654*x2-115.61676226989507654)*x2+26.680791293052710245)*x2-0.80850882706220339458);
}

inline const tdouble hGBP9(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*(((236.79727580591284664*x2-292.51428187789235835)*x2+97.504760625964124188)*x2-7.5003662019972399122)*x;
}

inline const tdouble hGBP10(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*((((482.75583655425998586*x2-711.42965386943581052)*x2+313.86602376592753672)*x2-41.848803168790340123)*x2+0.80478467632289119038);
}

inline const tdouble hGBP11(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*((((980.8625826292104648*x2-1681.4787130786464786)*x2+929.23823617504149297)*x2-182.20357572059637619)*x2+9.110178786029818454)*x;
}

inline const tdouble hGBP12(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*(((((1987.8246517436657541*x2-3889.2221447158681258)*x2+2592.8147631439119323)*x2-682.31967451155583149)*x2+60.204677162784335565)*x2-0.80272902883712449196);
}

inline const tdouble hGBP13(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*(((((4020.5733204198645581*x2-8845.2613049237024825)*x2+6922.3784125489846701)*x2-2307.4594708496615567)*x2+303.61308826969229813)*x2-10.715756056577374622)*x;
}

inline const tdouble hGBP14(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*((((((8119.2937311108653375*x2-1.98471624538265605224e4)*x2+1.78624462084439037426e4)*x2-7248.5288961801352343)*x2+1294.3801600321669412)*x2-81.750325896768444522)*x2+0.80147378330165131022);
}

inline const tdouble hGBP15(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*((((((1.63757787548797678028e4*x2-4.40451980303662712686e4)*x2+4.48608498457434252487e4)*x2-2.1533207925956845429e4)*x2+4915.1887657075403695)*x2-468.11321578167053303)*x2+12.318768836359751617)*x;
}

inline const tdouble hGBP16(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*(((((((3.29943448025226171012e4*x2-9.68543670009534835117e4)*x2+1.10213590035567758605e5)*x2-6.12297722419820929645e4)*x2+1.71443362277549858845e4)*x2-2236.2177688376068545)*x2+106.48656042083841555)*x2-0.80065083023186778277);
}

inline const tdouble hGBP17(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*(((((((6.64214069602393283276e4*x2-2.11340840328034217237e5)*x2+2.6588041202559141675e5)*x2-1.68085317947212985018e5)*x2+5.60284393157376543968e4)*x2-9412.7778050439264916)*x2+682.08534819158887785)*x2-13.920109146767119412)*x;
}

inline const tdouble hGBP18(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*((((((((1.33618922649897343945e5*x2-4.58122020513933734037e5)*x2+6.31653088890423765406e5)*x2-4.48269934051268501207e5)*x2+1.73897819244026584784e5)*x2-3.60676958432055107551e4)*x2+3606.7695843205510755)*x2-134.41377332871618933)*x2+0.80008198409950115337);
}

inline const tdouble hGBP19(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*((((((((2.68637738824175670743e5*x2-9.87425202164537739009e5)*x2+1.48113780324680660851e6)*x2-1.16695705710354447365e6)*x2+5.17601920489475422073e5)*x2-1.28508063018076660228e5)*x2+1.66584526134543812077e4)*x2-951.91157791167893265)*x2+15.520297465951285787)*x;
}

inline const tdouble hGBP20(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*(((((((((5.39813401444661431015e5*x2-2.11772949797521065921e6)*x2+3.43415594266250357032e6)*x2-2.97626848364083608612e6)*x2+1.48813424182041804306e6)*x2-4.32038973431734309997e5)*x2+6.95235129660262027755e4)*x2-5517.7391242877947661)*x2+165.53217372863383616)*x2-0.79967233685330352255);
}

inline const tdouble hGBP21(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*(((((((((1.08424917368967551738e6*x2-4.52211240733986534178e6)*x2+7.8847088127977149561e6)*x2-7.45850833643027022481e6)*x2+4.15545464458257937804e6)*x2-1.3851515481941930484e6)*x2+2.68093848037585732527e5)*x2-2.77338463487157678173e4)*x2+1283.9743679961004545)*x2-17.119658239948005729)*x;
}

inline const tdouble hGBP22(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*((((((((((2.1769525640318132937e6*x2-9.61909272479173354805e6)*x2+1.79478193523553088307e7)*x2-1.84080198485695458949e7)*x2+1.13184446366204638034e7)*x2-4.2686705486682895571e6)*x2+9.70152397424611262977e5)*x2-1.25180954506401452818e5)*x2+8093.5961965345759381)*x2-199.84188139591546474)*x2+0.79936752558366186427);
}

inline const tdouble hGBP23(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*((((((((((4.36942712071143183857e6*x2-2.03906598966533504426e7)*x2+4.05442190968339890242e7)*x2-4.48293804647920504212e7)*x2+3.01736214666869565845e7)*x2-1.27218512129815276712e7)*x2+3.33191341292373370379e6)*x2-5.19259233182919502724e5)*x2+4.39695318421020565438e4)*x2-1684.6563924177032732)*x2+18.718404360196704062)*x;
}

inline const tdouble hGBP24(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*(((((((((((8.76745316452157311141e6*x2-4.30910995958400815725e7)*x2+9.09700991467734873295e7)*x2-1.07894768755475535989e8)*x2+7.89473917722991853952e7)*x2-3.68421161604062765837e7)*x2+1.09530615612018667161e7)*x2-2.01178681736360816285e6)*x2+2.13371329114322084934e5)*x2-1.14715768341033381148e4)*x2+237.34296898144836518)*x2-0.79913457569511237022);
}

inline const tdouble hGBP25(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*(((((((((((1.75877702003494426608e7*x2-9.08103236875185519457e7)*x2+2.02874127387009561062e8)*x2-2.56973894690212100744e8)*x2+2.03188660917842119932e8)*x2-1.0407224095791913569e8)*x2+3.46907469859730452299e7)*x2-7.36676094296724908054e6)*x2+9.47154978381503373384e5)*x2-6.69705540269749908475e4)*x2+2160.3404524830643822)*x2-20.316681371314082583)*x;
}

inline const tdouble hGBP26(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*((((((((((((3.52735498979867249727e7*x2-1.90892152389104664326e8)*x2+4.49960073488603949547e8)*x2-6.06329177041381239891e8)*x2+5.15379800485173881054e8)*x2-2.87653842131259918213e8)*x2+1.06408535097335979342e8)*x2-2.57251403532021045685e7)*x2+3.91091660775031987578e6)*x2-3.47637031800028460566e5)*x2+1.5801683263637656637e4)*x2-278.03548264465086959)*x2+0.79895253633520357806);
}

inline const tdouble hGBP27(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*((((((((((((7.07293138742423206568e7*x2-4.00354606835334002972e8)*x2+9.93036426758230566978e8)*x2-1.41862346679747223854e9)*x2+1.29034368522536039352e9)*x2-7.79941071958439946175e8)*x2+3.17417878122620940208e8)*x2-8.62668797685171961784e7)*x2+1.52073025232962984592e7)*x2-1.64403270522122154944e6)*x2+9.86419623132732958766e4)*x2-2717.4094301177215129)*x2+21.91459217836872142)*x;
}

inline const tdouble hGBP28(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*(((((((((((((1.417982647985881567e8*x2-8.37898837446202993393e8)*x2+2.18169885976558542252e9)*x2-3.29393749415588378906e9)*x2+3.19310267290621423721e9)*x2-2.07891365512617301941e9)*x2+9.23961624500521540642e8)*x2-2.79337235314111113548e8)*x2+5.62081022278394326568e7)*x2-7.20616695228710677475e6)*x2+5.45331553146051359363e5)*x2-2.12466838888071943074e4)*x2+321.91945286071506871)*x2-0.7988075753367619436);
}

inline const tdouble hGBP29(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*(((((((((((((2.84231116494009375572e8*x2-1.75026529630521607399e9)*x2+4.77345080810513496399e9)*x2-7.59549090849433612823e9)*x2+7.81888769992063808441e9)*x2-5.4572644762711391449e9)*x2+2.63187222969104576111e9)*x2-8.77290743230348587036e8)*x2+1.98920575499904662371e8)*x2-2.96494082723435014486e7)*x2+2.73686845590863144025e6)*x2-1.41214342933860607445e5)*x2+3362.2462603300141382)*x2-23.512211610699399245)*x;
}

inline const tdouble hGBP30(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*((((((((((((((5.69650574260504007339e8*x2-3.64962571305882310867e9)*x2+1.04046347082817325592e10)*x2-1.74041162393076286316e10)*x2+1.89639191098116111755e10)*x2-1.41299789445655155182e10)*x2+7.35335638951878833771e9)*x2-2.68207527885183763504e9)*x2+6.7796902882088136673e8)*x2-1.156226250702278018e8)*x2+1.26902881174640282989e7)*x2-8.28270553121195291169e5)*x2+2.79821132811214629328e4)*x2-368.99490041039291555)*x2+0.79869026062855608128);
}

inline const tdouble hGBP31(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*((((((((((((((1.14153113920149755478e9*x2-7.59773184452144432068e9)*x2+2.26000328595510749817e10)*x2-3.96491804553527755737e10)*x2+4.55965575236556930542e10)*x2-3.61331210564818649292e10)*x2+2.01920382374457473755e10)*x2-8.00617259560531139374e9)*x2+2.23576628334722805023e9)*x2-4.30592024940947592258e8)*x2+5.50757241203537657857e7)*x2-4.39628839985085465014e6)*x2+1.9726935127535884385e5)*x2-4101.2339142486252968)*x2+25.109595393358929982)*x;
}

inline const tdouble hGBP32(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*(((((((((((((((2.28725523377816295624e9*x2-1.57929528046587486267e10)*x2+4.89322636078771057129e10)*x2-8.98473766811303100586e10)*x2+1.08762613877157745361e11)*x2-9.13605956568124847412e10)*x2+5.45865194175923614502e10)*x2-2.33942226075395851135e10)*x2+7.16149671659375e9)*x2-1.54065295794333672523e9)*x2+2.25962433831689357758e8)*x2-2.14974831340930685401e7)*x2+1.22343399950123135932e6)*x2-3.61962721745926464791e4)*x2+419.26183986014649463)*x2-0.79859398068599329878);
}

inline const tdouble hGBP33(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*(((((((((((((((4.58240890877765083313e9*x2-3.27818483474093513489e10)*x2+1.05630400230541229248e11)*x2-2.02602570933988983154e11)*x2+2.57545641017782623291e11)*x2-2.28628235710522766113e11)*x2+1.45490695452150848389e11)*x2-6.70590536989698104858e10)*x2+2.23530178996566009521e10)*x2-5.32214711896585845947e9)*x2+8.83249947402844429016e8)*x2-9.81388830447605103254e7)*x2+6.84689881707631424069e6)*x2-2.69765244200004788581e5)*x2+4940.7553882784768575)*x2-26.706785882586363101)*x;
}

inline const tdouble hGBP34(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*((((((((((((((((9.17972239370165252686e9*x2-6.7957347869791343689e10)*x2+2.27395740948917175293e11)*x2-4.54791481897834472656e11)*x2+6.05767342691787841797e11)*x2-5.66751818925197998047e11)*x2+3.82806053133686279297e11)*x2-1.88917272975066009521e11)*x2+6.81706197292101364136e10)*x2-1.78223842429307518005e10)*x2+3.30987135940142631531e9)*x2-4.22536769285288393497e8)*x2+3.52113974404407069087e7)*x2-1.76371937089863955043e6)*x2+4.60902274625080099213e4)*x2-472.72028166674880367)*x2+0.79851398930194050774);
}

inline const tdouble hGBP35(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*((((((((((((((((1.83876164240246124268e10*x2-1.40705238722970977783e11)*x2+4.88268179150608215332e11)*x2-1.0165993883853692627e12)*x2+1.41597771953676416016e12)*x2-1.39276497003616137695e12)*x2+9.95394173500420166016e11)*x2-5.23891670263379150391e11)*x2+2.03603353670540588379e11)*x2-5.80504320737809448242e10)*x2+1.19515595446019630432e10)*x2-1.72953922908896660805e9)*x2+1.6866074042534250021e8)*x2-1.03791224877133835107e7)*x2+3.62062412362094677519e5)*x2-5887.1936969446305739)*x2+28.303815850695336565)*x;
}

inline const tdouble hGBP36(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*(((((((((((((((((3.68285638078341217041e10*x2-2.90997525298520385742e11)*x2+1.04590414889902941895e12)*x2-2.26352390433372119141e12)*x2+3.29081552245441113281e12)*x2-3.39528585650058251953e12)*x2+2.56037949834470117188e12)*x2-1.43207666856568017578e12)*x2+5.96698611902366943359e11)*x2-1.84434116406186065674e11)*x2+4.1758667865551574707e10)*x2-6.77368765733546066284e9)*x2+7.60311879905000567436e8)*x2-5.59967832990589663386e7)*x2+2.48874592440262064338e6)*x2-5.78778121954097805428e4)*x2+529.37023349460162081)*x2-0.79844680768416542715);
}

inline const tdouble hGBP37(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*(((((((((((((((((7.37582355954528503418e10*x2-6.01180139442389648438e11)*x2+2.23537403961677294922e12)*x2-5.02149240783478125e12)*x2+7.6071862596303046875e12)*x2-8.21576116040072949219e12)*x2+6.52044536539740234375e12)*x2-3.86340205490759423828e12)*x2+1.71888650748007397461e12)*x2-5.72962169160024658203e11)*x2+1.4167791819229699707e11)*x2-2.55166061924377098083e10)*x2+3.25211647550676727295e9)*x2-2.8079498611125934124e8)*x2+1.53626436170293856412e7)*x2-4.77948912529802997597e5)*x2+6946.9318681657432535)*x2-29.900711053798033845)*x;
}

inline const tdouble hGBP38(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*((((((((((((((((((1.47708425040857757568e11*x2-1.24075077034320556641e12)*x2+4.76754234357902929688e12)*x2-1.11018826404469433594e13)*x2+1.74975324224435566406e13)*x2-1.9743484345324375e13)*x2+1.6452903621103640625e13)*x2-1.02970553274934335938e13)*x2+4.87422086198971972656e12)*x2-1.74407149487391137695e12)*x2+4.68145506518786560059e11)*x2-9.28553070781064605713e10)*x2+1.32859008555152282715e10)*x2-1.32257836570739865303e9)*x2+8.67580560595232397318e7)*x2-3.44571002789595769718e6)*x2+7.17856255811658047605e4)*x2-589.21170107660032045)*x2+0.79838983885718195488);
}

inline const tdouble hGBP39(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*((((((((((((((((((2.95781757599594238281e11*x2-2.55832013716012695312e12)*x2+1.0148003210735171875e13)*x2-2.44664186998546640625e13)*x2+4.00594531529310625e13)*x2-4.7142428927797140625e13)*x2+4.116167301904675e13)*x2-2.71395646279429023438e13)*x2+1.36236306564872109375e13)*x2-5.21122484127926269531e12)*x2+1.51037194552331152344e12)*x2-3.27608587864705566406e11)*x2+5.21195480693849716187e10)*x2-5.90032619653414916992e9)*x2+4.54506919901089787483e8)*x2-2.22615634237268492579e7)*x2+6.21665999864712357521e5)*x2-8126.3529394080069324)*x2+31.497492013209328121)*x;
}

inline const tdouble hGBP40(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*(((((((((((((((((((5.922580833797265625e11*x2-5.27034724830313769531e12)*x2+2.15605114703310195312e13)*x2-5.37575419326920078125e13)*x2+9.13141808171755e13)*x2-1.11892024381609421875e14)*x2+1.021622831310346875e14)*x2-7.0794759099331046875e13)*x2+3.75756798296449375e13)*x2-1.53086103009664589844e13)*x2+4.76825566751414355469e12)*x2-1.12410341622444360352e12)*x2+1.9721112565341116333e11)*x2-2.50995978104341430664e10)*x2+2.23257878098201274872e9)*x2-1.31328163587177246809e8)*x2+4.69029155668490100652e6)*x2-8.80530329790657269768e4)*x2+652.24468873382022593)*x2-0.79834111228129778226);
}

inline const tdouble hGBP41(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*(((((((((((((((((((1.18583979681550732422e12*x2-1.08482381412381601562e13)*x2+4.5727383557370984375e13)*x2-1.1778265461747071875e14)*x2+2.0729747212674853125e14)*x2-2.6409130010667965625e14)*x2+2.5169264751951159375e14)*x2-1.82907079253309625e14)*x2+1.023733652537180625e14)*x2-4.42742930071635390625e13)*x2+1.47580976690545175781e13)*x2-3.76100551625681347656e12)*x2+7.22453037021082885742e11)*x2-1.02371887836995544434e11)*x2+1.03701392873839664459e10)*x2-7.17431019881909370422e8)*x2+3.16513685242018960416e7)*x2-7.97933660273997229524e5)*x2+9431.8399559574136219)*x2-33.09417528406110165)*x;
}

inline const tdouble hGBP42(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*((((((((((((((((((((2.37420489648873925781e12*x2-2.23118050513399609375e13)*x2+9.6822215747481453125e13)*x2-2.5737551021482409375e14)*x2+4.687911078912869375e14)*x2-6.20054372037542125e14)*x2+6.15807424283860375e14)*x2-4.6836057621589375e14)*x2+2.7575577404015475e14)*x2-1.2621657319250865625e14)*x2+4.48554283191838515625e13)*x2-1.22980250802957167969e13)*x2+2.57048884875033447266e12)*x2-4.02162531747118835449e11)*x2+4.58606395851977539062e10)*x2-3.66885116681582021713e9)*x2+1.9469139446546214819e8)*x2-6.28761135528597421944e6)*x2+1.06932165906224079663e5)*x2-718.46919981785481468)*x2+0.79829911090872773816);
}

inline const tdouble hGBP43(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*((((((((((((((((((((4.75323296313567382812e12*x2-4.58547179973088515625e13)*x2+2.0468883154220403125e14)*x2-5.609990197823369375e14)*x2+1.056311445476235875e15)*x2-1.44865569665312375e15)*x2+1.49694421987489475e15)*x2-1.18935294181840925e15)*x2+7.349698636941225e14)*x2-3.5505790516624275e14)*x2+1.3407410448814840625e14)*x2-3.93784083112044296875e13)*x2+8.90702092753433398438e12)*x2-1.52755970510046582031e12)*x2+1.9418131844497442627e11)*x2-1.77147869458573188782e10)*x2+1.10717418411608242989e9)*x2-4.42378142377125173807e7)*x2+1.0119761426927697612e6)*x2-1.08697759687730358564e4)*x2+34.690774368424584395)*x;
}

inline const tdouble hGBP44(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*(((((((((((((((((((((9.51568744937636914062e12*x2-9.4172493033483375e13)*x2+4.320855562712768125e14)*x2-1.2199042010389865e15)*x2+2.3720359464646965e15)*x2-3.3688915594093535e15)*x2+3.616818643868051e15)*x2-2.9967925906335275e15)*x2+1.9397047932525225e15)*x2-9.86547821294318875e14)*x2+3.94619128517727625e14)*x2-1.23686592520481796875e14)*x2+3.01287853575532539062e13)*x2-5.62845440745500292969e12)*x2+7.9088352329578503418e11)*x2-8.13224865761767730713e10)*x2+5.88517994959173965454e9)*x2-2.83243954793185353279e8)*x2+8.31323976332200039178e6)*x2-1.2868792203284833522e5)*x2+787.88523693580600593)*x2-0.79826265140405894272);
}

inline const tdouble hGBP45(const tdouble& x)
{
	register const tdouble x2 = pow2(x);
	return (1-x2)*(((((((((((((((((((((1.90490232363655195312e13*x2-1.9327267395997821875e14)*x2+9.10825245098748125e14)*x2-2.6467510063457745e15)*x2+5.309446295862307e15)*x2-7.800297644538452e15)*x2+8.688939148346632e15)*x2-7.496023569538374e15)*x2+5.072309282054298e15)*x2-2.70986386301531e15)*x2+1.145012899865624e15)*x2-3.816709666218746875e14)*x2+9.9690177848997125e13)*x2-2.01739886534656875e13)*x2+3.11072840915117138672e12)*x2-3.5696883383701965332e11)*x2+2.94953061856859474182e10)*x2-1.67414018597804665565e9)*x2+6.08778249446562603116e7)*x2-1.26954749139799526893e6)*x2+1.24465440333136775735e4)*x2-36.287300388669621043)*x;
}



void hGBP_const(flxVec& r, const tdouble& x, const tdouble& f) {
	register const tdouble x2 = pow2(x);
	register const tdouble pf = (1-x2)*f;
	const tuint rs = r.get_N();
	tdouble* rp = &r[0];
	  rp[0] += ((1.0-x)/2.0)*f;
	  rp[1] += ((1.0+x)/2.0)*f;
	if (rs==2) return;
	  rp[2] += 0.96824583655185425535*pf;
	if (rs==3) return;
	  rp[3] += 2.5617376914898999374*x*pf;
	if (rs==4) return;
	  rp[4] += (5.8696784409369477942*x2-0.83852549156242117689)*pf;
	if (rs==5) return;
	  rp[5] += (12.744484100974821317*x2-4.2481613669916074016)*x*pf;
	if (rs==6) return;
	  rp[6] += ((26.941097860077082515*x2-14.695144287314771958)*x2+0.81639690485082072602)*pf;
	if (rs==7) return;
	  rp[7] += ((56.082367403612245482*x2-43.140282618163269035)*x2+5.8827658115677179396)*x*pf;
	if (rs==8) return;
	  rp[8] += (((115.61676226989507654*x2-115.61676226989507654)*x2+26.680791293052710245)*x2-0.80850882706220339458)*pf;
	if (rs==9) return;
	  rp[9] += (((236.79727580591284664*x2-292.51428187789235835)*x2+97.504760625964124188)*x2-7.5003662019972399122)*x*pf;
	if (rs==10) return;
	  rp[10] += ((((482.75583655425998586*x2-711.42965386943581052)*x2+313.86602376592753672)*x2-41.848803168790340123)*x2+0.80478467632289119038)*pf;
	if (rs==11) return;
	  rp[11] += ((((980.8625826292104648*x2-1681.4787130786464786)*x2+929.23823617504149297)*x2-182.20357572059637619)*x2+9.110178786029818454)*x*pf;
	if (rs==12) return;
	  rp[12] += (((((1987.8246517436657541*x2-3889.2221447158681258)*x2+2592.8147631439119323)*x2-682.31967451155583149)*x2+60.204677162784335565)*x2-0.80272902883712449196)*pf;
	if (rs==13) return;
	  rp[13] += (((((4020.5733204198645581*x2-8845.2613049237024825)*x2+6922.3784125489846701)*x2-2307.4594708496615567)*x2+303.61308826969229813)*x2-10.715756056577374622)*x*pf;
	if (rs==14) return;
	  rp[14] += ((((((8119.2937311108653375*x2-1.98471624538265605224e4)*x2+1.78624462084439037426e4)*x2-7248.5288961801352343)*x2+1294.3801600321669412)*x2-81.750325896768444522)*x2+0.80147378330165131022)*pf;
	if (rs==15) return;
	  rp[15] += ((((((1.63757787548797678028e4*x2-4.40451980303662712686e4)*x2+4.48608498457434252487e4)*x2-2.1533207925956845429e4)*x2+4915.1887657075403695)*x2-468.11321578167053303)*x2+12.318768836359751617)*x*pf;
	if (rs==16) return;
	  rp[16] += (((((((3.29943448025226171012e4*x2-9.68543670009534835117e4)*x2+1.10213590035567758605e5)*x2-6.12297722419820929645e4)*x2+1.71443362277549858845e4)*x2-2236.2177688376068545)*x2+106.48656042083841555)*x2-0.80065083023186778277)*pf;
	if (rs==17) return;
	  rp[17] += (((((((6.64214069602393283276e4*x2-2.11340840328034217237e5)*x2+2.6588041202559141675e5)*x2-1.68085317947212985018e5)*x2+5.60284393157376543968e4)*x2-9412.7778050439264916)*x2+682.08534819158887785)*x2-13.920109146767119412)*x*pf;
	if (rs==18) return;
	  rp[18] += ((((((((1.33618922649897343945e5*x2-4.58122020513933734037e5)*x2+6.31653088890423765406e5)*x2-4.48269934051268501207e5)*x2+1.73897819244026584784e5)*x2-3.60676958432055107551e4)*x2+3606.7695843205510755)*x2-134.41377332871618933)*x2+0.80008198409950115337)*pf;
	if (rs==19) return;
	  rp[19] += ((((((((2.68637738824175670743e5*x2-9.87425202164537739009e5)*x2+1.48113780324680660851e6)*x2-1.16695705710354447365e6)*x2+5.17601920489475422073e5)*x2-1.28508063018076660228e5)*x2+1.66584526134543812077e4)*x2-951.91157791167893265)*x2+15.520297465951285787)*x*pf;
	if (rs==20) return;
	  rp[20] += (((((((((5.39813401444661431015e5*x2-2.11772949797521065921e6)*x2+3.43415594266250357032e6)*x2-2.97626848364083608612e6)*x2+1.48813424182041804306e6)*x2-4.32038973431734309997e5)*x2+6.95235129660262027755e4)*x2-5517.7391242877947661)*x2+165.53217372863383616)*x2-0.79967233685330352255)*pf;
	if (rs==21) return;
	  rp[21] += (((((((((1.08424917368967551738e6*x2-4.52211240733986534178e6)*x2+7.8847088127977149561e6)*x2-7.45850833643027022481e6)*x2+4.15545464458257937804e6)*x2-1.3851515481941930484e6)*x2+2.68093848037585732527e5)*x2-2.77338463487157678173e4)*x2+1283.9743679961004545)*x2-17.119658239948005729)*x*pf;
	if (rs==22) return;
	  rp[22] += ((((((((((2.1769525640318132937e6*x2-9.61909272479173354805e6)*x2+1.79478193523553088307e7)*x2-1.84080198485695458949e7)*x2+1.13184446366204638034e7)*x2-4.2686705486682895571e6)*x2+9.70152397424611262977e5)*x2-1.25180954506401452818e5)*x2+8093.5961965345759381)*x2-199.84188139591546474)*x2+0.79936752558366186427)*pf;
	if (rs==23) return;
	  rp[23] += ((((((((((4.36942712071143183857e6*x2-2.03906598966533504426e7)*x2+4.05442190968339890242e7)*x2-4.48293804647920504212e7)*x2+3.01736214666869565845e7)*x2-1.27218512129815276712e7)*x2+3.33191341292373370379e6)*x2-5.19259233182919502724e5)*x2+4.39695318421020565438e4)*x2-1684.6563924177032732)*x2+18.718404360196704062)*x*pf;
	if (rs==24) return;
	  rp[24] += (((((((((((8.76745316452157311141e6*x2-4.30910995958400815725e7)*x2+9.09700991467734873295e7)*x2-1.07894768755475535989e8)*x2+7.89473917722991853952e7)*x2-3.68421161604062765837e7)*x2+1.09530615612018667161e7)*x2-2.01178681736360816285e6)*x2+2.13371329114322084934e5)*x2-1.14715768341033381148e4)*x2+237.34296898144836518)*x2-0.79913457569511237022)*pf;
	if (rs==25) return;
	  rp[25] += (((((((((((1.75877702003494426608e7*x2-9.08103236875185519457e7)*x2+2.02874127387009561062e8)*x2-2.56973894690212100744e8)*x2+2.03188660917842119932e8)*x2-1.0407224095791913569e8)*x2+3.46907469859730452299e7)*x2-7.36676094296724908054e6)*x2+9.47154978381503373384e5)*x2-6.69705540269749908475e4)*x2+2160.3404524830643822)*x2-20.316681371314082583)*x*pf;
	if (rs==26) return;
	  rp[26] += ((((((((((((3.52735498979867249727e7*x2-1.90892152389104664326e8)*x2+4.49960073488603949547e8)*x2-6.06329177041381239891e8)*x2+5.15379800485173881054e8)*x2-2.87653842131259918213e8)*x2+1.06408535097335979342e8)*x2-2.57251403532021045685e7)*x2+3.91091660775031987578e6)*x2-3.47637031800028460566e5)*x2+1.5801683263637656637e4)*x2-278.03548264465086959)*x2+0.79895253633520357806)*pf;
	if (rs==27) return;
	  rp[27] += ((((((((((((7.07293138742423206568e7*x2-4.00354606835334002972e8)*x2+9.93036426758230566978e8)*x2-1.41862346679747223854e9)*x2+1.29034368522536039352e9)*x2-7.79941071958439946175e8)*x2+3.17417878122620940208e8)*x2-8.62668797685171961784e7)*x2+1.52073025232962984592e7)*x2-1.64403270522122154944e6)*x2+9.86419623132732958766e4)*x2-2717.4094301177215129)*x2+21.91459217836872142)*x*pf;
	if (rs==28) return;
	  rp[28] += (((((((((((((1.417982647985881567e8*x2-8.37898837446202993393e8)*x2+2.18169885976558542252e9)*x2-3.29393749415588378906e9)*x2+3.19310267290621423721e9)*x2-2.07891365512617301941e9)*x2+9.23961624500521540642e8)*x2-2.79337235314111113548e8)*x2+5.62081022278394326568e7)*x2-7.20616695228710677475e6)*x2+5.45331553146051359363e5)*x2-2.12466838888071943074e4)*x2+321.91945286071506871)*x2-0.7988075753367619436)*pf;
	if (rs==29) return;
	  rp[29] += (((((((((((((2.84231116494009375572e8*x2-1.75026529630521607399e9)*x2+4.77345080810513496399e9)*x2-7.59549090849433612823e9)*x2+7.81888769992063808441e9)*x2-5.4572644762711391449e9)*x2+2.63187222969104576111e9)*x2-8.77290743230348587036e8)*x2+1.98920575499904662371e8)*x2-2.96494082723435014486e7)*x2+2.73686845590863144025e6)*x2-1.41214342933860607445e5)*x2+3362.2462603300141382)*x2-23.512211610699399245)*x*pf;
	if (rs==30) return;
	  rp[30] += ((((((((((((((5.69650574260504007339e8*x2-3.64962571305882310867e9)*x2+1.04046347082817325592e10)*x2-1.74041162393076286316e10)*x2+1.89639191098116111755e10)*x2-1.41299789445655155182e10)*x2+7.35335638951878833771e9)*x2-2.68207527885183763504e9)*x2+6.7796902882088136673e8)*x2-1.156226250702278018e8)*x2+1.26902881174640282989e7)*x2-8.28270553121195291169e5)*x2+2.79821132811214629328e4)*x2-368.99490041039291555)*x2+0.79869026062855608128)*pf;
	if (rs==31) return;
	  rp[31] += ((((((((((((((1.14153113920149755478e9*x2-7.59773184452144432068e9)*x2+2.26000328595510749817e10)*x2-3.96491804553527755737e10)*x2+4.55965575236556930542e10)*x2-3.61331210564818649292e10)*x2+2.01920382374457473755e10)*x2-8.00617259560531139374e9)*x2+2.23576628334722805023e9)*x2-4.30592024940947592258e8)*x2+5.50757241203537657857e7)*x2-4.39628839985085465014e6)*x2+1.9726935127535884385e5)*x2-4101.2339142486252968)*x2+25.109595393358929982)*x*pf;
	if (rs==32) return;
	  rp[32] += (((((((((((((((2.28725523377816295624e9*x2-1.57929528046587486267e10)*x2+4.89322636078771057129e10)*x2-8.98473766811303100586e10)*x2+1.08762613877157745361e11)*x2-9.13605956568124847412e10)*x2+5.45865194175923614502e10)*x2-2.33942226075395851135e10)*x2+7.16149671659375e9)*x2-1.54065295794333672523e9)*x2+2.25962433831689357758e8)*x2-2.14974831340930685401e7)*x2+1.22343399950123135932e6)*x2-3.61962721745926464791e4)*x2+419.26183986014649463)*x2-0.79859398068599329878)*pf;
	if (rs==33) return;
	  rp[33] += (((((((((((((((4.58240890877765083313e9*x2-3.27818483474093513489e10)*x2+1.05630400230541229248e11)*x2-2.02602570933988983154e11)*x2+2.57545641017782623291e11)*x2-2.28628235710522766113e11)*x2+1.45490695452150848389e11)*x2-6.70590536989698104858e10)*x2+2.23530178996566009521e10)*x2-5.32214711896585845947e9)*x2+8.83249947402844429016e8)*x2-9.81388830447605103254e7)*x2+6.84689881707631424069e6)*x2-2.69765244200004788581e5)*x2+4940.7553882784768575)*x2-26.706785882586363101)*x*pf;
	if (rs==34) return;
	  rp[34] += ((((((((((((((((9.17972239370165252686e9*x2-6.7957347869791343689e10)*x2+2.27395740948917175293e11)*x2-4.54791481897834472656e11)*x2+6.05767342691787841797e11)*x2-5.66751818925197998047e11)*x2+3.82806053133686279297e11)*x2-1.88917272975066009521e11)*x2+6.81706197292101364136e10)*x2-1.78223842429307518005e10)*x2+3.30987135940142631531e9)*x2-4.22536769285288393497e8)*x2+3.52113974404407069087e7)*x2-1.76371937089863955043e6)*x2+4.60902274625080099213e4)*x2-472.72028166674880367)*x2+0.79851398930194050774)*pf;
	if (rs==35) return;
	  rp[35] += ((((((((((((((((1.83876164240246124268e10*x2-1.40705238722970977783e11)*x2+4.88268179150608215332e11)*x2-1.0165993883853692627e12)*x2+1.41597771953676416016e12)*x2-1.39276497003616137695e12)*x2+9.95394173500420166016e11)*x2-5.23891670263379150391e11)*x2+2.03603353670540588379e11)*x2-5.80504320737809448242e10)*x2+1.19515595446019630432e10)*x2-1.72953922908896660805e9)*x2+1.6866074042534250021e8)*x2-1.03791224877133835107e7)*x2+3.62062412362094677519e5)*x2-5887.1936969446305739)*x2+28.303815850695336565)*x*pf;
	if (rs==36) return;
	  rp[36] += (((((((((((((((((3.68285638078341217041e10*x2-2.90997525298520385742e11)*x2+1.04590414889902941895e12)*x2-2.26352390433372119141e12)*x2+3.29081552245441113281e12)*x2-3.39528585650058251953e12)*x2+2.56037949834470117188e12)*x2-1.43207666856568017578e12)*x2+5.96698611902366943359e11)*x2-1.84434116406186065674e11)*x2+4.1758667865551574707e10)*x2-6.77368765733546066284e9)*x2+7.60311879905000567436e8)*x2-5.59967832990589663386e7)*x2+2.48874592440262064338e6)*x2-5.78778121954097805428e4)*x2+529.37023349460162081)*x2-0.79844680768416542715)*pf;
	if (rs==37) return;
	  rp[37] += (((((((((((((((((7.37582355954528503418e10*x2-6.01180139442389648438e11)*x2+2.23537403961677294922e12)*x2-5.02149240783478125e12)*x2+7.6071862596303046875e12)*x2-8.21576116040072949219e12)*x2+6.52044536539740234375e12)*x2-3.86340205490759423828e12)*x2+1.71888650748007397461e12)*x2-5.72962169160024658203e11)*x2+1.4167791819229699707e11)*x2-2.55166061924377098083e10)*x2+3.25211647550676727295e9)*x2-2.8079498611125934124e8)*x2+1.53626436170293856412e7)*x2-4.77948912529802997597e5)*x2+6946.9318681657432535)*x2-29.900711053798033845)*x*pf;
	if (rs==38) return;
	  rp[38] += ((((((((((((((((((1.47708425040857757568e11*x2-1.24075077034320556641e12)*x2+4.76754234357902929688e12)*x2-1.11018826404469433594e13)*x2+1.74975324224435566406e13)*x2-1.9743484345324375e13)*x2+1.6452903621103640625e13)*x2-1.02970553274934335938e13)*x2+4.87422086198971972656e12)*x2-1.74407149487391137695e12)*x2+4.68145506518786560059e11)*x2-9.28553070781064605713e10)*x2+1.32859008555152282715e10)*x2-1.32257836570739865303e9)*x2+8.67580560595232397318e7)*x2-3.44571002789595769718e6)*x2+7.17856255811658047605e4)*x2-589.21170107660032045)*x2+0.79838983885718195488)*pf;
	if (rs==39) return;
	  rp[39] += ((((((((((((((((((2.95781757599594238281e11*x2-2.55832013716012695312e12)*x2+1.0148003210735171875e13)*x2-2.44664186998546640625e13)*x2+4.00594531529310625e13)*x2-4.7142428927797140625e13)*x2+4.116167301904675e13)*x2-2.71395646279429023438e13)*x2+1.36236306564872109375e13)*x2-5.21122484127926269531e12)*x2+1.51037194552331152344e12)*x2-3.27608587864705566406e11)*x2+5.21195480693849716187e10)*x2-5.90032619653414916992e9)*x2+4.54506919901089787483e8)*x2-2.22615634237268492579e7)*x2+6.21665999864712357521e5)*x2-8126.3529394080069324)*x2+31.497492013209328121)*x*pf;
	if (rs==40) return;
	  rp[40] += (((((((((((((((((((5.922580833797265625e11*x2-5.27034724830313769531e12)*x2+2.15605114703310195312e13)*x2-5.37575419326920078125e13)*x2+9.13141808171755e13)*x2-1.11892024381609421875e14)*x2+1.021622831310346875e14)*x2-7.0794759099331046875e13)*x2+3.75756798296449375e13)*x2-1.53086103009664589844e13)*x2+4.76825566751414355469e12)*x2-1.12410341622444360352e12)*x2+1.9721112565341116333e11)*x2-2.50995978104341430664e10)*x2+2.23257878098201274872e9)*x2-1.31328163587177246809e8)*x2+4.69029155668490100652e6)*x2-8.80530329790657269768e4)*x2+652.24468873382022593)*x2-0.79834111228129778226)*pf;
	if (rs==41) return;
	  rp[41] += (((((((((((((((((((1.18583979681550732422e12*x2-1.08482381412381601562e13)*x2+4.5727383557370984375e13)*x2-1.1778265461747071875e14)*x2+2.0729747212674853125e14)*x2-2.6409130010667965625e14)*x2+2.5169264751951159375e14)*x2-1.82907079253309625e14)*x2+1.023733652537180625e14)*x2-4.42742930071635390625e13)*x2+1.47580976690545175781e13)*x2-3.76100551625681347656e12)*x2+7.22453037021082885742e11)*x2-1.02371887836995544434e11)*x2+1.03701392873839664459e10)*x2-7.17431019881909370422e8)*x2+3.16513685242018960416e7)*x2-7.97933660273997229524e5)*x2+9431.8399559574136219)*x2-33.09417528406110165)*x*pf;
	if (rs==42) return;
	  rp[42] += ((((((((((((((((((((2.37420489648873925781e12*x2-2.23118050513399609375e13)*x2+9.6822215747481453125e13)*x2-2.5737551021482409375e14)*x2+4.687911078912869375e14)*x2-6.20054372037542125e14)*x2+6.15807424283860375e14)*x2-4.6836057621589375e14)*x2+2.7575577404015475e14)*x2-1.2621657319250865625e14)*x2+4.48554283191838515625e13)*x2-1.22980250802957167969e13)*x2+2.57048884875033447266e12)*x2-4.02162531747118835449e11)*x2+4.58606395851977539062e10)*x2-3.66885116681582021713e9)*x2+1.9469139446546214819e8)*x2-6.28761135528597421944e6)*x2+1.06932165906224079663e5)*x2-718.46919981785481468)*x2+0.79829911090872773816)*pf;
	if (rs==43) return;
	  rp[43] += ((((((((((((((((((((4.75323296313567382812e12*x2-4.58547179973088515625e13)*x2+2.0468883154220403125e14)*x2-5.609990197823369375e14)*x2+1.056311445476235875e15)*x2-1.44865569665312375e15)*x2+1.49694421987489475e15)*x2-1.18935294181840925e15)*x2+7.349698636941225e14)*x2-3.5505790516624275e14)*x2+1.3407410448814840625e14)*x2-3.93784083112044296875e13)*x2+8.90702092753433398438e12)*x2-1.52755970510046582031e12)*x2+1.9418131844497442627e11)*x2-1.77147869458573188782e10)*x2+1.10717418411608242989e9)*x2-4.42378142377125173807e7)*x2+1.0119761426927697612e6)*x2-1.08697759687730358564e4)*x2+34.690774368424584395)*x*pf;
	if (rs==44) return;
	  rp[44] += (((((((((((((((((((((9.51568744937636914062e12*x2-9.4172493033483375e13)*x2+4.320855562712768125e14)*x2-1.2199042010389865e15)*x2+2.3720359464646965e15)*x2-3.3688915594093535e15)*x2+3.616818643868051e15)*x2-2.9967925906335275e15)*x2+1.9397047932525225e15)*x2-9.86547821294318875e14)*x2+3.94619128517727625e14)*x2-1.23686592520481796875e14)*x2+3.01287853575532539062e13)*x2-5.62845440745500292969e12)*x2+7.9088352329578503418e11)*x2-8.13224865761767730713e10)*x2+5.88517994959173965454e9)*x2-2.83243954793185353279e8)*x2+8.31323976332200039178e6)*x2-1.2868792203284833522e5)*x2+787.88523693580600593)*x2-0.79826265140405894272)*pf;
	if (rs==45) return;
	  rp[45] += (((((((((((((((((((((1.90490232363655195312e13*x2-1.9327267395997821875e14)*x2+9.10825245098748125e14)*x2-2.6467510063457745e15)*x2+5.309446295862307e15)*x2-7.800297644538452e15)*x2+8.688939148346632e15)*x2-7.496023569538374e15)*x2+5.072309282054298e15)*x2-2.70986386301531e15)*x2+1.145012899865624e15)*x2-3.816709666218746875e14)*x2+9.9690177848997125e13)*x2-2.01739886534656875e13)*x2+3.11072840915117138672e12)*x2-3.5696883383701965332e11)*x2+2.94953061856859474182e10)*x2-1.67414018597804665565e9)*x2+6.08778249446562603116e7)*x2-1.26954749139799526893e6)*x2+1.24465440333136775735e4)*x2-36.287300388669621043)*x*pf;
}


void hGBP_vec(flxVec& r, const tdouble& x) {
	register const tdouble x2 = pow2(x);
	register const tdouble pf = (1-x2);
	const tuint rs = r.get_N();
	tdouble* rp = &r[0];
	  rp[0] = (1.0-x)/2.0;
	  rp[1] = (1.0+x)/2.0;
	if (rs==2) return;
	  rp[2] = pf*0.96824583655185425535;
	if (rs==3) return;
	  rp[3] = pf*2.5617376914898999374*x;
	if (rs==4) return;
	  rp[4] = pf*(5.8696784409369477942*x2-0.83852549156242117689);
	if (rs==5) return;
	  rp[5] = pf*(12.744484100974821317*x2-4.2481613669916074016)*x;
	if (rs==6) return;
	  rp[6] = pf*((26.941097860077082515*x2-14.695144287314771958)*x2+0.81639690485082072602);
	if (rs==7) return;
	  rp[7] = pf*((56.082367403612245482*x2-43.140282618163269035)*x2+5.8827658115677179396)*x;
	if (rs==8) return;
	  rp[8] = pf*(((115.61676226989507654*x2-115.61676226989507654)*x2+26.680791293052710245)*x2-0.80850882706220339458);
	if (rs==9) return;
	  rp[9] = pf*(((236.79727580591284664*x2-292.51428187789235835)*x2+97.504760625964124188)*x2-7.5003662019972399122)*x;
	if (rs==10) return;
	  rp[10] = pf*((((482.75583655425998586*x2-711.42965386943581052)*x2+313.86602376592753672)*x2-41.848803168790340123)*x2+0.80478467632289119038);
	if (rs==11) return;
	  rp[11] = pf*((((980.8625826292104648*x2-1681.4787130786464786)*x2+929.23823617504149297)*x2-182.20357572059637619)*x2+9.110178786029818454)*x;
	if (rs==12) return;
	  rp[12] = pf*(((((1987.8246517436657541*x2-3889.2221447158681258)*x2+2592.8147631439119323)*x2-682.31967451155583149)*x2+60.204677162784335565)*x2-0.80272902883712449196);
	if (rs==13) return;
	  rp[13] = pf*(((((4020.5733204198645581*x2-8845.2613049237024825)*x2+6922.3784125489846701)*x2-2307.4594708496615567)*x2+303.61308826969229813)*x2-10.715756056577374622)*x;
	if (rs==14) return;
	  rp[14] = pf*((((((8119.2937311108653375*x2-1.98471624538265605224e4)*x2+1.78624462084439037426e4)*x2-7248.5288961801352343)*x2+1294.3801600321669412)*x2-81.750325896768444522)*x2+0.80147378330165131022);
	if (rs==15) return;
	  rp[15] = pf*((((((1.63757787548797678028e4*x2-4.40451980303662712686e4)*x2+4.48608498457434252487e4)*x2-2.1533207925956845429e4)*x2+4915.1887657075403695)*x2-468.11321578167053303)*x2+12.318768836359751617)*x;
	if (rs==16) return;
	  rp[16] = pf*(((((((3.29943448025226171012e4*x2-9.68543670009534835117e4)*x2+1.10213590035567758605e5)*x2-6.12297722419820929645e4)*x2+1.71443362277549858845e4)*x2-2236.2177688376068545)*x2+106.48656042083841555)*x2-0.80065083023186778277);
	if (rs==17) return;
	  rp[17] = pf*(((((((6.64214069602393283276e4*x2-2.11340840328034217237e5)*x2+2.6588041202559141675e5)*x2-1.68085317947212985018e5)*x2+5.60284393157376543968e4)*x2-9412.7778050439264916)*x2+682.08534819158887785)*x2-13.920109146767119412)*x;
	if (rs==18) return;
	  rp[18] = pf*((((((((1.33618922649897343945e5*x2-4.58122020513933734037e5)*x2+6.31653088890423765406e5)*x2-4.48269934051268501207e5)*x2+1.73897819244026584784e5)*x2-3.60676958432055107551e4)*x2+3606.7695843205510755)*x2-134.41377332871618933)*x2+0.80008198409950115337);
	if (rs==19) return;
	  rp[19] = pf*((((((((2.68637738824175670743e5*x2-9.87425202164537739009e5)*x2+1.48113780324680660851e6)*x2-1.16695705710354447365e6)*x2+5.17601920489475422073e5)*x2-1.28508063018076660228e5)*x2+1.66584526134543812077e4)*x2-951.91157791167893265)*x2+15.520297465951285787)*x;
	if (rs==20) return;
	  rp[20] = pf*(((((((((5.39813401444661431015e5*x2-2.11772949797521065921e6)*x2+3.43415594266250357032e6)*x2-2.97626848364083608612e6)*x2+1.48813424182041804306e6)*x2-4.32038973431734309997e5)*x2+6.95235129660262027755e4)*x2-5517.7391242877947661)*x2+165.53217372863383616)*x2-0.79967233685330352255);
	if (rs==21) return;
	  rp[21] = pf*(((((((((1.08424917368967551738e6*x2-4.52211240733986534178e6)*x2+7.8847088127977149561e6)*x2-7.45850833643027022481e6)*x2+4.15545464458257937804e6)*x2-1.3851515481941930484e6)*x2+2.68093848037585732527e5)*x2-2.77338463487157678173e4)*x2+1283.9743679961004545)*x2-17.119658239948005729)*x;
	if (rs==22) return;
	  rp[22] = pf*((((((((((2.1769525640318132937e6*x2-9.61909272479173354805e6)*x2+1.79478193523553088307e7)*x2-1.84080198485695458949e7)*x2+1.13184446366204638034e7)*x2-4.2686705486682895571e6)*x2+9.70152397424611262977e5)*x2-1.25180954506401452818e5)*x2+8093.5961965345759381)*x2-199.84188139591546474)*x2+0.79936752558366186427);
	if (rs==23) return;
	  rp[23] = pf*((((((((((4.36942712071143183857e6*x2-2.03906598966533504426e7)*x2+4.05442190968339890242e7)*x2-4.48293804647920504212e7)*x2+3.01736214666869565845e7)*x2-1.27218512129815276712e7)*x2+3.33191341292373370379e6)*x2-5.19259233182919502724e5)*x2+4.39695318421020565438e4)*x2-1684.6563924177032732)*x2+18.718404360196704062)*x;
	if (rs==24) return;
	  rp[24] = pf*(((((((((((8.76745316452157311141e6*x2-4.30910995958400815725e7)*x2+9.09700991467734873295e7)*x2-1.07894768755475535989e8)*x2+7.89473917722991853952e7)*x2-3.68421161604062765837e7)*x2+1.09530615612018667161e7)*x2-2.01178681736360816285e6)*x2+2.13371329114322084934e5)*x2-1.14715768341033381148e4)*x2+237.34296898144836518)*x2-0.79913457569511237022);
	if (rs==25) return;
	  rp[25] = pf*(((((((((((1.75877702003494426608e7*x2-9.08103236875185519457e7)*x2+2.02874127387009561062e8)*x2-2.56973894690212100744e8)*x2+2.03188660917842119932e8)*x2-1.0407224095791913569e8)*x2+3.46907469859730452299e7)*x2-7.36676094296724908054e6)*x2+9.47154978381503373384e5)*x2-6.69705540269749908475e4)*x2+2160.3404524830643822)*x2-20.316681371314082583)*x;
	if (rs==26) return;
	  rp[26] = pf*((((((((((((3.52735498979867249727e7*x2-1.90892152389104664326e8)*x2+4.49960073488603949547e8)*x2-6.06329177041381239891e8)*x2+5.15379800485173881054e8)*x2-2.87653842131259918213e8)*x2+1.06408535097335979342e8)*x2-2.57251403532021045685e7)*x2+3.91091660775031987578e6)*x2-3.47637031800028460566e5)*x2+1.5801683263637656637e4)*x2-278.03548264465086959)*x2+0.79895253633520357806);
	if (rs==27) return;
	  rp[27] = pf*((((((((((((7.07293138742423206568e7*x2-4.00354606835334002972e8)*x2+9.93036426758230566978e8)*x2-1.41862346679747223854e9)*x2+1.29034368522536039352e9)*x2-7.79941071958439946175e8)*x2+3.17417878122620940208e8)*x2-8.62668797685171961784e7)*x2+1.52073025232962984592e7)*x2-1.64403270522122154944e6)*x2+9.86419623132732958766e4)*x2-2717.4094301177215129)*x2+21.91459217836872142)*x;
	if (rs==28) return;
	  rp[28] = pf*(((((((((((((1.417982647985881567e8*x2-8.37898837446202993393e8)*x2+2.18169885976558542252e9)*x2-3.29393749415588378906e9)*x2+3.19310267290621423721e9)*x2-2.07891365512617301941e9)*x2+9.23961624500521540642e8)*x2-2.79337235314111113548e8)*x2+5.62081022278394326568e7)*x2-7.20616695228710677475e6)*x2+5.45331553146051359363e5)*x2-2.12466838888071943074e4)*x2+321.91945286071506871)*x2-0.7988075753367619436);
	if (rs==29) return;
	  rp[29] = pf*(((((((((((((2.84231116494009375572e8*x2-1.75026529630521607399e9)*x2+4.77345080810513496399e9)*x2-7.59549090849433612823e9)*x2+7.81888769992063808441e9)*x2-5.4572644762711391449e9)*x2+2.63187222969104576111e9)*x2-8.77290743230348587036e8)*x2+1.98920575499904662371e8)*x2-2.96494082723435014486e7)*x2+2.73686845590863144025e6)*x2-1.41214342933860607445e5)*x2+3362.2462603300141382)*x2-23.512211610699399245)*x;
	if (rs==30) return;
	  rp[30] = pf*((((((((((((((5.69650574260504007339e8*x2-3.64962571305882310867e9)*x2+1.04046347082817325592e10)*x2-1.74041162393076286316e10)*x2+1.89639191098116111755e10)*x2-1.41299789445655155182e10)*x2+7.35335638951878833771e9)*x2-2.68207527885183763504e9)*x2+6.7796902882088136673e8)*x2-1.156226250702278018e8)*x2+1.26902881174640282989e7)*x2-8.28270553121195291169e5)*x2+2.79821132811214629328e4)*x2-368.99490041039291555)*x2+0.79869026062855608128);
	if (rs==31) return;
	  rp[31] = pf*((((((((((((((1.14153113920149755478e9*x2-7.59773184452144432068e9)*x2+2.26000328595510749817e10)*x2-3.96491804553527755737e10)*x2+4.55965575236556930542e10)*x2-3.61331210564818649292e10)*x2+2.01920382374457473755e10)*x2-8.00617259560531139374e9)*x2+2.23576628334722805023e9)*x2-4.30592024940947592258e8)*x2+5.50757241203537657857e7)*x2-4.39628839985085465014e6)*x2+1.9726935127535884385e5)*x2-4101.2339142486252968)*x2+25.109595393358929982)*x;
	if (rs==32) return;
	  rp[32] = pf*(((((((((((((((2.28725523377816295624e9*x2-1.57929528046587486267e10)*x2+4.89322636078771057129e10)*x2-8.98473766811303100586e10)*x2+1.08762613877157745361e11)*x2-9.13605956568124847412e10)*x2+5.45865194175923614502e10)*x2-2.33942226075395851135e10)*x2+7.16149671659375e9)*x2-1.54065295794333672523e9)*x2+2.25962433831689357758e8)*x2-2.14974831340930685401e7)*x2+1.22343399950123135932e6)*x2-3.61962721745926464791e4)*x2+419.26183986014649463)*x2-0.79859398068599329878);
	if (rs==33) return;
	  rp[33] = pf*(((((((((((((((4.58240890877765083313e9*x2-3.27818483474093513489e10)*x2+1.05630400230541229248e11)*x2-2.02602570933988983154e11)*x2+2.57545641017782623291e11)*x2-2.28628235710522766113e11)*x2+1.45490695452150848389e11)*x2-6.70590536989698104858e10)*x2+2.23530178996566009521e10)*x2-5.32214711896585845947e9)*x2+8.83249947402844429016e8)*x2-9.81388830447605103254e7)*x2+6.84689881707631424069e6)*x2-2.69765244200004788581e5)*x2+4940.7553882784768575)*x2-26.706785882586363101)*x;
	if (rs==34) return;
	  rp[34] = pf*((((((((((((((((9.17972239370165252686e9*x2-6.7957347869791343689e10)*x2+2.27395740948917175293e11)*x2-4.54791481897834472656e11)*x2+6.05767342691787841797e11)*x2-5.66751818925197998047e11)*x2+3.82806053133686279297e11)*x2-1.88917272975066009521e11)*x2+6.81706197292101364136e10)*x2-1.78223842429307518005e10)*x2+3.30987135940142631531e9)*x2-4.22536769285288393497e8)*x2+3.52113974404407069087e7)*x2-1.76371937089863955043e6)*x2+4.60902274625080099213e4)*x2-472.72028166674880367)*x2+0.79851398930194050774);
	if (rs==35) return;
	  rp[35] = pf*((((((((((((((((1.83876164240246124268e10*x2-1.40705238722970977783e11)*x2+4.88268179150608215332e11)*x2-1.0165993883853692627e12)*x2+1.41597771953676416016e12)*x2-1.39276497003616137695e12)*x2+9.95394173500420166016e11)*x2-5.23891670263379150391e11)*x2+2.03603353670540588379e11)*x2-5.80504320737809448242e10)*x2+1.19515595446019630432e10)*x2-1.72953922908896660805e9)*x2+1.6866074042534250021e8)*x2-1.03791224877133835107e7)*x2+3.62062412362094677519e5)*x2-5887.1936969446305739)*x2+28.303815850695336565)*x;
	if (rs==36) return;
	  rp[36] = pf*(((((((((((((((((3.68285638078341217041e10*x2-2.90997525298520385742e11)*x2+1.04590414889902941895e12)*x2-2.26352390433372119141e12)*x2+3.29081552245441113281e12)*x2-3.39528585650058251953e12)*x2+2.56037949834470117188e12)*x2-1.43207666856568017578e12)*x2+5.96698611902366943359e11)*x2-1.84434116406186065674e11)*x2+4.1758667865551574707e10)*x2-6.77368765733546066284e9)*x2+7.60311879905000567436e8)*x2-5.59967832990589663386e7)*x2+2.48874592440262064338e6)*x2-5.78778121954097805428e4)*x2+529.37023349460162081)*x2-0.79844680768416542715);
	if (rs==37) return;
	  rp[37] = pf*(((((((((((((((((7.37582355954528503418e10*x2-6.01180139442389648438e11)*x2+2.23537403961677294922e12)*x2-5.02149240783478125e12)*x2+7.6071862596303046875e12)*x2-8.21576116040072949219e12)*x2+6.52044536539740234375e12)*x2-3.86340205490759423828e12)*x2+1.71888650748007397461e12)*x2-5.72962169160024658203e11)*x2+1.4167791819229699707e11)*x2-2.55166061924377098083e10)*x2+3.25211647550676727295e9)*x2-2.8079498611125934124e8)*x2+1.53626436170293856412e7)*x2-4.77948912529802997597e5)*x2+6946.9318681657432535)*x2-29.900711053798033845)*x;
	if (rs==38) return;
	  rp[38] = pf*((((((((((((((((((1.47708425040857757568e11*x2-1.24075077034320556641e12)*x2+4.76754234357902929688e12)*x2-1.11018826404469433594e13)*x2+1.74975324224435566406e13)*x2-1.9743484345324375e13)*x2+1.6452903621103640625e13)*x2-1.02970553274934335938e13)*x2+4.87422086198971972656e12)*x2-1.74407149487391137695e12)*x2+4.68145506518786560059e11)*x2-9.28553070781064605713e10)*x2+1.32859008555152282715e10)*x2-1.32257836570739865303e9)*x2+8.67580560595232397318e7)*x2-3.44571002789595769718e6)*x2+7.17856255811658047605e4)*x2-589.21170107660032045)*x2+0.79838983885718195488);
	if (rs==39) return;
	  rp[39] = pf*((((((((((((((((((2.95781757599594238281e11*x2-2.55832013716012695312e12)*x2+1.0148003210735171875e13)*x2-2.44664186998546640625e13)*x2+4.00594531529310625e13)*x2-4.7142428927797140625e13)*x2+4.116167301904675e13)*x2-2.71395646279429023438e13)*x2+1.36236306564872109375e13)*x2-5.21122484127926269531e12)*x2+1.51037194552331152344e12)*x2-3.27608587864705566406e11)*x2+5.21195480693849716187e10)*x2-5.90032619653414916992e9)*x2+4.54506919901089787483e8)*x2-2.22615634237268492579e7)*x2+6.21665999864712357521e5)*x2-8126.3529394080069324)*x2+31.497492013209328121)*x;
	if (rs==40) return;
	  rp[40] = pf*(((((((((((((((((((5.922580833797265625e11*x2-5.27034724830313769531e12)*x2+2.15605114703310195312e13)*x2-5.37575419326920078125e13)*x2+9.13141808171755e13)*x2-1.11892024381609421875e14)*x2+1.021622831310346875e14)*x2-7.0794759099331046875e13)*x2+3.75756798296449375e13)*x2-1.53086103009664589844e13)*x2+4.76825566751414355469e12)*x2-1.12410341622444360352e12)*x2+1.9721112565341116333e11)*x2-2.50995978104341430664e10)*x2+2.23257878098201274872e9)*x2-1.31328163587177246809e8)*x2+4.69029155668490100652e6)*x2-8.80530329790657269768e4)*x2+652.24468873382022593)*x2-0.79834111228129778226);
	if (rs==41) return;
	  rp[41] = pf*(((((((((((((((((((1.18583979681550732422e12*x2-1.08482381412381601562e13)*x2+4.5727383557370984375e13)*x2-1.1778265461747071875e14)*x2+2.0729747212674853125e14)*x2-2.6409130010667965625e14)*x2+2.5169264751951159375e14)*x2-1.82907079253309625e14)*x2+1.023733652537180625e14)*x2-4.42742930071635390625e13)*x2+1.47580976690545175781e13)*x2-3.76100551625681347656e12)*x2+7.22453037021082885742e11)*x2-1.02371887836995544434e11)*x2+1.03701392873839664459e10)*x2-7.17431019881909370422e8)*x2+3.16513685242018960416e7)*x2-7.97933660273997229524e5)*x2+9431.8399559574136219)*x2-33.09417528406110165)*x;
	if (rs==42) return;
	  rp[42] = pf*((((((((((((((((((((2.37420489648873925781e12*x2-2.23118050513399609375e13)*x2+9.6822215747481453125e13)*x2-2.5737551021482409375e14)*x2+4.687911078912869375e14)*x2-6.20054372037542125e14)*x2+6.15807424283860375e14)*x2-4.6836057621589375e14)*x2+2.7575577404015475e14)*x2-1.2621657319250865625e14)*x2+4.48554283191838515625e13)*x2-1.22980250802957167969e13)*x2+2.57048884875033447266e12)*x2-4.02162531747118835449e11)*x2+4.58606395851977539062e10)*x2-3.66885116681582021713e9)*x2+1.9469139446546214819e8)*x2-6.28761135528597421944e6)*x2+1.06932165906224079663e5)*x2-718.46919981785481468)*x2+0.79829911090872773816);
	if (rs==43) return;
	  rp[43] = pf*((((((((((((((((((((4.75323296313567382812e12*x2-4.58547179973088515625e13)*x2+2.0468883154220403125e14)*x2-5.609990197823369375e14)*x2+1.056311445476235875e15)*x2-1.44865569665312375e15)*x2+1.49694421987489475e15)*x2-1.18935294181840925e15)*x2+7.349698636941225e14)*x2-3.5505790516624275e14)*x2+1.3407410448814840625e14)*x2-3.93784083112044296875e13)*x2+8.90702092753433398438e12)*x2-1.52755970510046582031e12)*x2+1.9418131844497442627e11)*x2-1.77147869458573188782e10)*x2+1.10717418411608242989e9)*x2-4.42378142377125173807e7)*x2+1.0119761426927697612e6)*x2-1.08697759687730358564e4)*x2+34.690774368424584395)*x;
	if (rs==44) return;
	  rp[44] = pf*(((((((((((((((((((((9.51568744937636914062e12*x2-9.4172493033483375e13)*x2+4.320855562712768125e14)*x2-1.2199042010389865e15)*x2+2.3720359464646965e15)*x2-3.3688915594093535e15)*x2+3.616818643868051e15)*x2-2.9967925906335275e15)*x2+1.9397047932525225e15)*x2-9.86547821294318875e14)*x2+3.94619128517727625e14)*x2-1.23686592520481796875e14)*x2+3.01287853575532539062e13)*x2-5.62845440745500292969e12)*x2+7.9088352329578503418e11)*x2-8.13224865761767730713e10)*x2+5.88517994959173965454e9)*x2-2.83243954793185353279e8)*x2+8.31323976332200039178e6)*x2-1.2868792203284833522e5)*x2+787.88523693580600593)*x2-0.79826265140405894272);
	if (rs==45) return;
	  rp[45] = pf*(((((((((((((((((((((1.90490232363655195312e13*x2-1.9327267395997821875e14)*x2+9.10825245098748125e14)*x2-2.6467510063457745e15)*x2+5.309446295862307e15)*x2-7.800297644538452e15)*x2+8.688939148346632e15)*x2-7.496023569538374e15)*x2+5.072309282054298e15)*x2-2.70986386301531e15)*x2+1.145012899865624e15)*x2-3.816709666218746875e14)*x2+9.9690177848997125e13)*x2-2.01739886534656875e13)*x2+3.11072840915117138672e12)*x2-3.5696883383701965332e11)*x2+2.94953061856859474182e10)*x2-1.67414018597804665565e9)*x2+6.08778249446562603116e7)*x2-1.26954749139799526893e6)*x2+1.24465440333136775735e4)*x2-36.287300388669621043)*x;
}


const tdouble hGBP_rlz(const tuint N, const tdouble& x, const tdouble& c0, const tdouble& c1, const tdouble* coordH)
{
	register const tdouble x2 = pow2(x);
	register const tdouble pf = (1-x2);
	tdouble r = 0.0;
	  r += ((1.0-x)/2.0)*c0;
	  r += ((1.0+x)/2.0)*c1;
	if (N==1) return r;
	  r += pf*0.96824583655185425535*coordH[0];
	if (N==2) return r;
	  r += pf*2.5617376914898999374*x*coordH[1];
	if (N==3) return r;
	  r += pf*(5.8696784409369477942*x2-0.83852549156242117689)*coordH[2];
	if (N==4) return r;
	  r += pf*(12.744484100974821317*x2-4.2481613669916074016)*x*coordH[3];
	if (N==5) return r;
	  r += pf*((26.941097860077082515*x2-14.695144287314771958)*x2+0.81639690485082072602)*coordH[4];
	if (N==6) return r;
	  r += pf*((56.082367403612245482*x2-43.140282618163269035)*x2+5.8827658115677179396)*x*coordH[5];
	if (N==7) return r;
	  r += pf*(((115.61676226989507654*x2-115.61676226989507654)*x2+26.680791293052710245)*x2-0.80850882706220339458)*coordH[6];
	if (N==8) return r;
	  r += pf*(((236.79727580591284664*x2-292.51428187789235835)*x2+97.504760625964124188)*x2-7.5003662019972399122)*x*coordH[7];
	if (N==9) return r;
	  r += pf*((((482.75583655425998586*x2-711.42965386943581052)*x2+313.86602376592753672)*x2-41.848803168790340123)*x2+0.80478467632289119038)*coordH[8];
	if (N==10) return r;
	  r += pf*((((980.8625826292104648*x2-1681.4787130786464786)*x2+929.23823617504149297)*x2-182.20357572059637619)*x2+9.110178786029818454)*x*coordH[9];
	if (N==11) return r;
	  r += pf*(((((1987.8246517436657541*x2-3889.2221447158681258)*x2+2592.8147631439119323)*x2-682.31967451155583149)*x2+60.204677162784335565)*x2-0.80272902883712449196)*coordH[10];
	if (N==12) return r;
	  r += pf*(((((4020.5733204198645581*x2-8845.2613049237024825)*x2+6922.3784125489846701)*x2-2307.4594708496615567)*x2+303.61308826969229813)*x2-10.715756056577374622)*x*coordH[11];
	if (N==13) return r;
	  r += pf*((((((8119.2937311108653375*x2-1.98471624538265605224e4)*x2+1.78624462084439037426e4)*x2-7248.5288961801352343)*x2+1294.3801600321669412)*x2-81.750325896768444522)*x2+0.80147378330165131022)*coordH[12];
	if (N==14) return r;
	  r += pf*((((((1.63757787548797678028e4*x2-4.40451980303662712686e4)*x2+4.48608498457434252487e4)*x2-2.1533207925956845429e4)*x2+4915.1887657075403695)*x2-468.11321578167053303)*x2+12.318768836359751617)*x*coordH[13];
	if (N==15) return r;
	  r += pf*(((((((3.29943448025226171012e4*x2-9.68543670009534835117e4)*x2+1.10213590035567758605e5)*x2-6.12297722419820929645e4)*x2+1.71443362277549858845e4)*x2-2236.2177688376068545)*x2+106.48656042083841555)*x2-0.80065083023186778277)*coordH[14];
	if (N==16) return r;
	  r += pf*(((((((6.64214069602393283276e4*x2-2.11340840328034217237e5)*x2+2.6588041202559141675e5)*x2-1.68085317947212985018e5)*x2+5.60284393157376543968e4)*x2-9412.7778050439264916)*x2+682.08534819158887785)*x2-13.920109146767119412)*x*coordH[15];
	if (N==17) return r;
	  r += pf*((((((((1.33618922649897343945e5*x2-4.58122020513933734037e5)*x2+6.31653088890423765406e5)*x2-4.48269934051268501207e5)*x2+1.73897819244026584784e5)*x2-3.60676958432055107551e4)*x2+3606.7695843205510755)*x2-134.41377332871618933)*x2+0.80008198409950115337)*coordH[16];
	if (N==18) return r;
	  r += pf*((((((((2.68637738824175670743e5*x2-9.87425202164537739009e5)*x2+1.48113780324680660851e6)*x2-1.16695705710354447365e6)*x2+5.17601920489475422073e5)*x2-1.28508063018076660228e5)*x2+1.66584526134543812077e4)*x2-951.91157791167893265)*x2+15.520297465951285787)*x*coordH[17];
	if (N==19) return r;
	  r += pf*(((((((((5.39813401444661431015e5*x2-2.11772949797521065921e6)*x2+3.43415594266250357032e6)*x2-2.97626848364083608612e6)*x2+1.48813424182041804306e6)*x2-4.32038973431734309997e5)*x2+6.95235129660262027755e4)*x2-5517.7391242877947661)*x2+165.53217372863383616)*x2-0.79967233685330352255)*coordH[18];
	if (N==20) return r;
	  r += pf*(((((((((1.08424917368967551738e6*x2-4.52211240733986534178e6)*x2+7.8847088127977149561e6)*x2-7.45850833643027022481e6)*x2+4.15545464458257937804e6)*x2-1.3851515481941930484e6)*x2+2.68093848037585732527e5)*x2-2.77338463487157678173e4)*x2+1283.9743679961004545)*x2-17.119658239948005729)*x*coordH[19];
	if (N==21) return r;
	  r += pf*((((((((((2.1769525640318132937e6*x2-9.61909272479173354805e6)*x2+1.79478193523553088307e7)*x2-1.84080198485695458949e7)*x2+1.13184446366204638034e7)*x2-4.2686705486682895571e6)*x2+9.70152397424611262977e5)*x2-1.25180954506401452818e5)*x2+8093.5961965345759381)*x2-199.84188139591546474)*x2+0.79936752558366186427)*coordH[20];
	if (N==22) return r;
	  r += pf*((((((((((4.36942712071143183857e6*x2-2.03906598966533504426e7)*x2+4.05442190968339890242e7)*x2-4.48293804647920504212e7)*x2+3.01736214666869565845e7)*x2-1.27218512129815276712e7)*x2+3.33191341292373370379e6)*x2-5.19259233182919502724e5)*x2+4.39695318421020565438e4)*x2-1684.6563924177032732)*x2+18.718404360196704062)*x*coordH[21];
	if (N==23) return r;
	  r += pf*(((((((((((8.76745316452157311141e6*x2-4.30910995958400815725e7)*x2+9.09700991467734873295e7)*x2-1.07894768755475535989e8)*x2+7.89473917722991853952e7)*x2-3.68421161604062765837e7)*x2+1.09530615612018667161e7)*x2-2.01178681736360816285e6)*x2+2.13371329114322084934e5)*x2-1.14715768341033381148e4)*x2+237.34296898144836518)*x2-0.79913457569511237022)*coordH[22];
	if (N==24) return r;
	  r += pf*(((((((((((1.75877702003494426608e7*x2-9.08103236875185519457e7)*x2+2.02874127387009561062e8)*x2-2.56973894690212100744e8)*x2+2.03188660917842119932e8)*x2-1.0407224095791913569e8)*x2+3.46907469859730452299e7)*x2-7.36676094296724908054e6)*x2+9.47154978381503373384e5)*x2-6.69705540269749908475e4)*x2+2160.3404524830643822)*x2-20.316681371314082583)*x*coordH[23];
	if (N==25) return r;
	  r += pf*((((((((((((3.52735498979867249727e7*x2-1.90892152389104664326e8)*x2+4.49960073488603949547e8)*x2-6.06329177041381239891e8)*x2+5.15379800485173881054e8)*x2-2.87653842131259918213e8)*x2+1.06408535097335979342e8)*x2-2.57251403532021045685e7)*x2+3.91091660775031987578e6)*x2-3.47637031800028460566e5)*x2+1.5801683263637656637e4)*x2-278.03548264465086959)*x2+0.79895253633520357806)*coordH[24];
	if (N==26) return r;
	  r += pf*((((((((((((7.07293138742423206568e7*x2-4.00354606835334002972e8)*x2+9.93036426758230566978e8)*x2-1.41862346679747223854e9)*x2+1.29034368522536039352e9)*x2-7.79941071958439946175e8)*x2+3.17417878122620940208e8)*x2-8.62668797685171961784e7)*x2+1.52073025232962984592e7)*x2-1.64403270522122154944e6)*x2+9.86419623132732958766e4)*x2-2717.4094301177215129)*x2+21.91459217836872142)*x*coordH[25];
	if (N==27) return r;
	  r += pf*(((((((((((((1.417982647985881567e8*x2-8.37898837446202993393e8)*x2+2.18169885976558542252e9)*x2-3.29393749415588378906e9)*x2+3.19310267290621423721e9)*x2-2.07891365512617301941e9)*x2+9.23961624500521540642e8)*x2-2.79337235314111113548e8)*x2+5.62081022278394326568e7)*x2-7.20616695228710677475e6)*x2+5.45331553146051359363e5)*x2-2.12466838888071943074e4)*x2+321.91945286071506871)*x2-0.7988075753367619436)*coordH[26];
	if (N==28) return r;
	  r += pf*(((((((((((((2.84231116494009375572e8*x2-1.75026529630521607399e9)*x2+4.77345080810513496399e9)*x2-7.59549090849433612823e9)*x2+7.81888769992063808441e9)*x2-5.4572644762711391449e9)*x2+2.63187222969104576111e9)*x2-8.77290743230348587036e8)*x2+1.98920575499904662371e8)*x2-2.96494082723435014486e7)*x2+2.73686845590863144025e6)*x2-1.41214342933860607445e5)*x2+3362.2462603300141382)*x2-23.512211610699399245)*x*coordH[27];
	if (N==29) return r;
	  r += pf*((((((((((((((5.69650574260504007339e8*x2-3.64962571305882310867e9)*x2+1.04046347082817325592e10)*x2-1.74041162393076286316e10)*x2+1.89639191098116111755e10)*x2-1.41299789445655155182e10)*x2+7.35335638951878833771e9)*x2-2.68207527885183763504e9)*x2+6.7796902882088136673e8)*x2-1.156226250702278018e8)*x2+1.26902881174640282989e7)*x2-8.28270553121195291169e5)*x2+2.79821132811214629328e4)*x2-368.99490041039291555)*x2+0.79869026062855608128)*coordH[28];
	if (N==30) return r;
	  r += pf*((((((((((((((1.14153113920149755478e9*x2-7.59773184452144432068e9)*x2+2.26000328595510749817e10)*x2-3.96491804553527755737e10)*x2+4.55965575236556930542e10)*x2-3.61331210564818649292e10)*x2+2.01920382374457473755e10)*x2-8.00617259560531139374e9)*x2+2.23576628334722805023e9)*x2-4.30592024940947592258e8)*x2+5.50757241203537657857e7)*x2-4.39628839985085465014e6)*x2+1.9726935127535884385e5)*x2-4101.2339142486252968)*x2+25.109595393358929982)*x*coordH[29];
	if (N==31) return r;
	  r += pf*(((((((((((((((2.28725523377816295624e9*x2-1.57929528046587486267e10)*x2+4.89322636078771057129e10)*x2-8.98473766811303100586e10)*x2+1.08762613877157745361e11)*x2-9.13605956568124847412e10)*x2+5.45865194175923614502e10)*x2-2.33942226075395851135e10)*x2+7.16149671659375e9)*x2-1.54065295794333672523e9)*x2+2.25962433831689357758e8)*x2-2.14974831340930685401e7)*x2+1.22343399950123135932e6)*x2-3.61962721745926464791e4)*x2+419.26183986014649463)*x2-0.79859398068599329878)*coordH[30];
	if (N==32) return r;
	  r += pf*(((((((((((((((4.58240890877765083313e9*x2-3.27818483474093513489e10)*x2+1.05630400230541229248e11)*x2-2.02602570933988983154e11)*x2+2.57545641017782623291e11)*x2-2.28628235710522766113e11)*x2+1.45490695452150848389e11)*x2-6.70590536989698104858e10)*x2+2.23530178996566009521e10)*x2-5.32214711896585845947e9)*x2+8.83249947402844429016e8)*x2-9.81388830447605103254e7)*x2+6.84689881707631424069e6)*x2-2.69765244200004788581e5)*x2+4940.7553882784768575)*x2-26.706785882586363101)*x*coordH[31];
	if (N==33) return r;
	  r += pf*((((((((((((((((9.17972239370165252686e9*x2-6.7957347869791343689e10)*x2+2.27395740948917175293e11)*x2-4.54791481897834472656e11)*x2+6.05767342691787841797e11)*x2-5.66751818925197998047e11)*x2+3.82806053133686279297e11)*x2-1.88917272975066009521e11)*x2+6.81706197292101364136e10)*x2-1.78223842429307518005e10)*x2+3.30987135940142631531e9)*x2-4.22536769285288393497e8)*x2+3.52113974404407069087e7)*x2-1.76371937089863955043e6)*x2+4.60902274625080099213e4)*x2-472.72028166674880367)*x2+0.79851398930194050774)*coordH[32];
	if (N==34) return r;
	  r += pf*((((((((((((((((1.83876164240246124268e10*x2-1.40705238722970977783e11)*x2+4.88268179150608215332e11)*x2-1.0165993883853692627e12)*x2+1.41597771953676416016e12)*x2-1.39276497003616137695e12)*x2+9.95394173500420166016e11)*x2-5.23891670263379150391e11)*x2+2.03603353670540588379e11)*x2-5.80504320737809448242e10)*x2+1.19515595446019630432e10)*x2-1.72953922908896660805e9)*x2+1.6866074042534250021e8)*x2-1.03791224877133835107e7)*x2+3.62062412362094677519e5)*x2-5887.1936969446305739)*x2+28.303815850695336565)*x*coordH[33];
	if (N==35) return r;
	  r += pf*(((((((((((((((((3.68285638078341217041e10*x2-2.90997525298520385742e11)*x2+1.04590414889902941895e12)*x2-2.26352390433372119141e12)*x2+3.29081552245441113281e12)*x2-3.39528585650058251953e12)*x2+2.56037949834470117188e12)*x2-1.43207666856568017578e12)*x2+5.96698611902366943359e11)*x2-1.84434116406186065674e11)*x2+4.1758667865551574707e10)*x2-6.77368765733546066284e9)*x2+7.60311879905000567436e8)*x2-5.59967832990589663386e7)*x2+2.48874592440262064338e6)*x2-5.78778121954097805428e4)*x2+529.37023349460162081)*x2-0.79844680768416542715)*coordH[34];
	if (N==36) return r;
	  r += pf*(((((((((((((((((7.37582355954528503418e10*x2-6.01180139442389648438e11)*x2+2.23537403961677294922e12)*x2-5.02149240783478125e12)*x2+7.6071862596303046875e12)*x2-8.21576116040072949219e12)*x2+6.52044536539740234375e12)*x2-3.86340205490759423828e12)*x2+1.71888650748007397461e12)*x2-5.72962169160024658203e11)*x2+1.4167791819229699707e11)*x2-2.55166061924377098083e10)*x2+3.25211647550676727295e9)*x2-2.8079498611125934124e8)*x2+1.53626436170293856412e7)*x2-4.77948912529802997597e5)*x2+6946.9318681657432535)*x2-29.900711053798033845)*x*coordH[35];
	if (N==37) return r;
	  r += pf*((((((((((((((((((1.47708425040857757568e11*x2-1.24075077034320556641e12)*x2+4.76754234357902929688e12)*x2-1.11018826404469433594e13)*x2+1.74975324224435566406e13)*x2-1.9743484345324375e13)*x2+1.6452903621103640625e13)*x2-1.02970553274934335938e13)*x2+4.87422086198971972656e12)*x2-1.74407149487391137695e12)*x2+4.68145506518786560059e11)*x2-9.28553070781064605713e10)*x2+1.32859008555152282715e10)*x2-1.32257836570739865303e9)*x2+8.67580560595232397318e7)*x2-3.44571002789595769718e6)*x2+7.17856255811658047605e4)*x2-589.21170107660032045)*x2+0.79838983885718195488)*coordH[36];
	if (N==38) return r;
	  r += pf*((((((((((((((((((2.95781757599594238281e11*x2-2.55832013716012695312e12)*x2+1.0148003210735171875e13)*x2-2.44664186998546640625e13)*x2+4.00594531529310625e13)*x2-4.7142428927797140625e13)*x2+4.116167301904675e13)*x2-2.71395646279429023438e13)*x2+1.36236306564872109375e13)*x2-5.21122484127926269531e12)*x2+1.51037194552331152344e12)*x2-3.27608587864705566406e11)*x2+5.21195480693849716187e10)*x2-5.90032619653414916992e9)*x2+4.54506919901089787483e8)*x2-2.22615634237268492579e7)*x2+6.21665999864712357521e5)*x2-8126.3529394080069324)*x2+31.497492013209328121)*x*coordH[37];
	if (N==39) return r;
	  r += pf*(((((((((((((((((((5.922580833797265625e11*x2-5.27034724830313769531e12)*x2+2.15605114703310195312e13)*x2-5.37575419326920078125e13)*x2+9.13141808171755e13)*x2-1.11892024381609421875e14)*x2+1.021622831310346875e14)*x2-7.0794759099331046875e13)*x2+3.75756798296449375e13)*x2-1.53086103009664589844e13)*x2+4.76825566751414355469e12)*x2-1.12410341622444360352e12)*x2+1.9721112565341116333e11)*x2-2.50995978104341430664e10)*x2+2.23257878098201274872e9)*x2-1.31328163587177246809e8)*x2+4.69029155668490100652e6)*x2-8.80530329790657269768e4)*x2+652.24468873382022593)*x2-0.79834111228129778226)*coordH[38];
	if (N==40) return r;
	  r += pf*(((((((((((((((((((1.18583979681550732422e12*x2-1.08482381412381601562e13)*x2+4.5727383557370984375e13)*x2-1.1778265461747071875e14)*x2+2.0729747212674853125e14)*x2-2.6409130010667965625e14)*x2+2.5169264751951159375e14)*x2-1.82907079253309625e14)*x2+1.023733652537180625e14)*x2-4.42742930071635390625e13)*x2+1.47580976690545175781e13)*x2-3.76100551625681347656e12)*x2+7.22453037021082885742e11)*x2-1.02371887836995544434e11)*x2+1.03701392873839664459e10)*x2-7.17431019881909370422e8)*x2+3.16513685242018960416e7)*x2-7.97933660273997229524e5)*x2+9431.8399559574136219)*x2-33.09417528406110165)*x*coordH[39];
	if (N==41) return r;
	  r += pf*((((((((((((((((((((2.37420489648873925781e12*x2-2.23118050513399609375e13)*x2+9.6822215747481453125e13)*x2-2.5737551021482409375e14)*x2+4.687911078912869375e14)*x2-6.20054372037542125e14)*x2+6.15807424283860375e14)*x2-4.6836057621589375e14)*x2+2.7575577404015475e14)*x2-1.2621657319250865625e14)*x2+4.48554283191838515625e13)*x2-1.22980250802957167969e13)*x2+2.57048884875033447266e12)*x2-4.02162531747118835449e11)*x2+4.58606395851977539062e10)*x2-3.66885116681582021713e9)*x2+1.9469139446546214819e8)*x2-6.28761135528597421944e6)*x2+1.06932165906224079663e5)*x2-718.46919981785481468)*x2+0.79829911090872773816)*coordH[40];
	if (N==42) return r;
	  r += pf*((((((((((((((((((((4.75323296313567382812e12*x2-4.58547179973088515625e13)*x2+2.0468883154220403125e14)*x2-5.609990197823369375e14)*x2+1.056311445476235875e15)*x2-1.44865569665312375e15)*x2+1.49694421987489475e15)*x2-1.18935294181840925e15)*x2+7.349698636941225e14)*x2-3.5505790516624275e14)*x2+1.3407410448814840625e14)*x2-3.93784083112044296875e13)*x2+8.90702092753433398438e12)*x2-1.52755970510046582031e12)*x2+1.9418131844497442627e11)*x2-1.77147869458573188782e10)*x2+1.10717418411608242989e9)*x2-4.42378142377125173807e7)*x2+1.0119761426927697612e6)*x2-1.08697759687730358564e4)*x2+34.690774368424584395)*x*coordH[41];
	if (N==43) return r;
	  r += pf*(((((((((((((((((((((9.51568744937636914062e12*x2-9.4172493033483375e13)*x2+4.320855562712768125e14)*x2-1.2199042010389865e15)*x2+2.3720359464646965e15)*x2-3.3688915594093535e15)*x2+3.616818643868051e15)*x2-2.9967925906335275e15)*x2+1.9397047932525225e15)*x2-9.86547821294318875e14)*x2+3.94619128517727625e14)*x2-1.23686592520481796875e14)*x2+3.01287853575532539062e13)*x2-5.62845440745500292969e12)*x2+7.9088352329578503418e11)*x2-8.13224865761767730713e10)*x2+5.88517994959173965454e9)*x2-2.83243954793185353279e8)*x2+8.31323976332200039178e6)*x2-1.2868792203284833522e5)*x2+787.88523693580600593)*x2-0.79826265140405894272)*coordH[42];
	if (N==44) return r;
	  r += pf*(((((((((((((((((((((1.90490232363655195312e13*x2-1.9327267395997821875e14)*x2+9.10825245098748125e14)*x2-2.6467510063457745e15)*x2+5.309446295862307e15)*x2-7.800297644538452e15)*x2+8.688939148346632e15)*x2-7.496023569538374e15)*x2+5.072309282054298e15)*x2-2.70986386301531e15)*x2+1.145012899865624e15)*x2-3.816709666218746875e14)*x2+9.9690177848997125e13)*x2-2.01739886534656875e13)*x2+3.11072840915117138672e12)*x2-3.5696883383701965332e11)*x2+2.94953061856859474182e10)*x2-1.67414018597804665565e9)*x2+6.08778249446562603116e7)*x2-1.26954749139799526893e6)*x2+1.24465440333136775735e4)*x2-36.287300388669621043)*x*coordH[43];
	if (N==45) return r;
	return r;
}

